eas<-function(n,m,xmin,xmax,xlow,xup,fn,maxeval=1500,ftol=1.e-07,ratio=0.99,pmut=0.9,beta=2,maxclimbs=5) {


# generate initial population

mvector <- 1:m

pop <- mat.or.vec( nr=m,nc=n ) # matrix of initial population

for (i in 1:n) { pop[,i] <- runif( n=m,min=xmin[i],max=xmax[i] ) }

fpop <- apply( pop,1,fn ) # fitness of initial population

temperature <- max( fpop ) - min( fpop ) # temperature of initial population

neval <- m # number of function evaluations

iter <- 0 # number of iterations taken by the algorithm

## Main loop ##

repeat {

iter <- iter+1 # number of iterations taken by the algorithm

# compute statistics of the population

meanpop <- apply( pop,2,mean ) # mean

sdpop <- apply( pop,2,sd ) # standard devation


# generate a simplex, selecting its vertices randomly from the actual population

sn <- sample( mvector,size=n+1 ) # coordinates of simplex vertex

s <- pop[sn,] # simplex

fns <- fpop[sn] # function value of each vertex of simplex


# determine the highest (worst) and the lowest (best) point of simplex

s1position <- which.min( fns )   # highest (worst) point of simplex

sn1position <- which.max( fns )  # lowest (best) point of simplex


# check temperature according to annealing schedule parameter, beta

if( temperature > beta*( max(fns)-min(fns) ) ) { temperature <- beta*( max(fns)-min(fns) ) }


# determine the randomized worst point, according to the criterio xw = max f(x) + rnd*T

gsfunction <- fns[-s1position]+runif( n,min=0,max=1 )*temperature # criterio xw = max f(x) + rnd*T

wposition <- which.max( gsfunction ) # coordinates of randomized worst point at simplex matrix (s)

wpopposition <- sn[-s1position][wposition] # coordinates of randomized worst point at population matrix (pop)

w <- s[-s1position,][wposition,] # randomized worst point

fnw <- fns[-s1position][wposition] # function value of randomized worst point


# Compute the centroid of the simplex

g <- ( apply( s,2,sum )-w )/n # centroid of the simplex


# make a reflection step

r0 <- g + ( 0.5 + runif(1) ) * (g - w) # reflection step

r0 <- ifelse( r0 < xup,ifelse( xlow < r0,r0,xlow ),xup ) # reset the reflection point to boundaries

fnr0 <- fn( r0 ) # function value of reflection step

neval <- neval+1 # number of function evaluations


# check if the number of function evaluations exceeded the maximum value

if( neval >= maxeval ) {

if( fnr0 < fnw ) { w <- r0 ; pop[wpopposition,] <- w ; fpop[wpopposition] <- fnw }

ftolpop <- 2*abs( max(fpop)-min(fpop) )/( abs(max(fpop))+abs(min(fpop)) ) # check convergence criteria

if( is.infinite( ftolpop ) ) { ftolpop <- 2*abs( max(fpop)-min(fpop) ) } # check convergence criteria

if( is.nan( ftolpop ) ) { ftolpop <- 0 } # check convergence criteria

break
                       }


# check if the reflection point is either not accepted (no move) or fr<fw (downhill move) the method follows the modified (quasi-stochastic) Nelder-Mead procedure, making contraction and expansion moves respectively

if ( fnr0 < fnw ) { # check if the reflected point is better than the randomized worst point of simplex (w)


if ( fnr0 < fns[s1position] ) { # the reflected point is better than the lowest (best) point of simplex (s1)

# line minimization employing subsequent random expansion steps

ns <- 1

r01 <- r0

fnr01 <- fnr0


repeat { # multiple expansion

ns <- ns + runif( 1,min=0,max=1 )

rs <- g + ns*(r0 - g) # expansion

neval <- neval+1 # number of function evaluations


if( any(rs > xup) || any(rs < xlow) ) {

rs <- ifelse( rs < xup,ifelse(xlow < rs,rs,xlow),xup )

fnrs <- fn(rs)

if( fnrs < fnr01 ) { w <- rs; fnw <- fnrs } else { w <- r01; fnw <- fnr01 }

break

                                      }


fnrs <- fn(rs) # function value of expansion step

if( fnrs < fnr01 ) {

 if( neval >= maxeval ) {

   w <- rs

   fnw <- fnrs

   break

                         } else {

   r01 <- rs

   fnr01 <- fnrs

                                }

                     } else {

w <- r01

fnw <- fnr01

break

                             }

       }


pop[wpopposition,] <- w

fpop[wpopposition] <- fnw

                               } else { # the reflected point is not better than the lowest (best) point of simplex (s1)

# outside contraction step between xc and xr

r1 <- g + ( 0.25 + 0.5*runif(1) ) * (r0 - g) # outside contraction

r1 <- ifelse( r1 < xup,ifelse(xlow < r1,r1,xlow),xup ) # reset the new point to boundaries

fnr1 <- fn(r1) # function value at outside contraction step

neval <- neval+1 # number of function evaluations

if ( fnr1 < fnr0 ) { w <- r1; fnw <- fnr1 } else { w <- r0; fnw <- fnr0 }

pop[wpopposition,] <- w

fpop[wpopposition] <- fnw

                                       }


                      } else { # the reflected point is not better than the randomized worst point of simplex (w)


if ( fnr0 - runif(1,min=0,max=1)*temperature > fnw + runif(1,min=0,max=1)*temperature ) { # dont't accept the reflection step and try an inside contraction step


temperature <- ratio*temperature # reduce temperature

r1 <- g - (0.25 + 0.5*runif(1))*(g-w) # inside contraction

r1 <- ifelse( r1 < xup,ifelse(xlow < r1,r1,xlow),xup ) # reset the new point to boundaries

fnr1 <- fn(r1) # function value at inside contraction step

neval <- neval+1

if ( fnr1 < fnw ) { # succesful inside contraction

w <- r1

fnw <- fnr1

pop[wpopposition,] <- w

fpop[wpopposition] <- fnw

                    } else { # unsuccesful inside contraction-multiple contraction step

mcon <- 0.5*(t(t(s[-s1position,])+s[s1position,])) # multiple contraction step

fmcon <- apply( mcon,1,fn ) # function values at multiple contraction steps

neval <- neval+n # number of function evaluations

pop[sn[-s1position],] <- mcon

fpop[sn[-s1position]] <- fmcon

                           }

                                                                                         } else { # accept reflection point and try some random uphill steps along the reflection direction


ns <- 1

r01 <- r0

fnr01 <- fnr0


for ( i in 1:maxclimbs ) { # uphill steps

ns <- ns+runif(1)

rs <- g + ns* (r0 - g)

neval <- neval+1

if( any(rs > xup) || any(rs < xlow) ) { rs <- ifelse( rs < xup,ifelse(xlow < rs,rs,xlow),xup ); fnrs <- fn(rs); if( fnrs < fnr01 ) { w <- rs; fnw <- fnrs }; break }

fnrs <- fn(rs) # function value of expansion step

if( fnrs < fnr01 ) { w <- rs; fnw <- fnrs ;break } else { if( neval >= maxeval ) { break } else { r01 <- rs; fnr01 <- fnrs } }

                       }


if( fnrs < fnr01 ) {

pop[wpopposition,] <- w

fpop[wpopposition] <- fnw


                       } else { # try a mutation step by generating a random point out of the range (xmean-xstdev, xmean+xstdev)

# mutation

samplen <- sample( c(-1,1),size=n,replace=TRUE )

a <- ifelse( xup < meanpop+sdpop,meanpop+sdpop,xup )

b <- ifelse( meanpop-sdpop < xlow,meanpop-sdpop,xlow )

newpoint <- ifelse( samplen>0,runif(n,min=meanpop+sdpop,max=a),runif(n,min=b,max=meanpop-sdpop) )

newpoint <- ifelse( newpoint < xup,ifelse(xlow < newpoint,newpoint,xlow),xup )

fnnewpoint <- fn( newpoint )

neval <- neval+1 # number of function evaluations

if( fnnewpoint < fnr0 ) { w <- newpoint; fnw <- fnnewpoint } else { if( runif(1,min=0,max=1 ) <= pmut) { w <- newpoint; fnw <- fnnewpoint } else { w <- r0; fnw <- fnr0 } }

pop[wpopposition,] <- w

fpop[wpopposition] <- fnw

                                          }

                                                                                                         }
                                                }


ftolpop <- 2*abs(max(fpop)-min(fpop))/(abs(max(fpop))+abs(min(fpop)))

if( is.nan(ftolpop) ) { ftolpop <- 0 }

if( is.infinite(ftolpop) ) { ftolpop <- 2*abs(max(fpop)-min(fpop)) }

if( ftolpop < ftol || neval >= maxeval ) { break } # check corvegence criteria

if( temperature > beta*(max(fpop) - min(fpop)) ) { temperature <- beta*(max(fpop) - min(fpop)) }

                    }


return( list( bestpar=pop[which.min(fpop),],bestval=min(fpop),nfeval=neval,niter=iter,ftolpop=ftolpop,pop=pop ) )

                                                               }
